class WordMemory extends WordQuiz {
	turn(click) {
		let target = click.currentTarget;
		if (!target.classList.contains('turn')) {
			// show an open card
			this.turned.push(target);
			put(target, '.turn');
		}
		else if (this.turned.length < 2) {
			return; // keep open
		}

		if (this.turned.length <= 1) {
			return; // first choice
		}

		// compare two cards
		let match = !this.pairs ? this.turned[0].id == this.turned[1].id : (
			this.pairs[this.turned[0].id] == this.turned[1].id
			|| this.pairs[this.turned[1].id] == this.turned[0].id
		);
		if (!match && !this.turned[0].classList.contains('bad')) {
			put(this.turned[0], '.bad'); // indicate failure on first card
			return;
		}

		if (match) {
			// lock both as correct
			this.turned.forEach(card => put(card, '.good![onclick]'));
			this.turned = [];
			if (Array.from(this.form.children).every(card => card.classList.contains('good'))) {
				put(this.form, '.good');
			}
			return;
		}

		// fold back earlier cards
		this.turned.splice(0, 2)
		.forEach(card => put(card, '!.turn!.bad'));
	}

	load(dataurl) {
		if (dataurl) {
			super.load(dataurl);
		}
		else {
			this.dataurl = '/data/wordpairs.json';
			fetch(this.dataurl).then(res => res.json()).then(pairs => {
				this.pairs = pairs;
				this.setup();
			});
		}
	}

	setup() {
		this.turned = [];
		this.form = document.getElementById('quiz');

		let cards;
		if (this.words) {
			const formstyle = window.getComputedStyle(this.form)
			const gridsize = [
				formstyle['grid-template-rows'], formstyle['grid-template-columns']
			].map(val => val.match(/ /g).length + 1).reduce((x, y) => x * y) / 2;
			cards = this.words.splice(0, gridsize || 6).map(row => row[2]);
			cards.push(...cards.map(val => -val));
		}
		else {
			cards = Object.entries(this.pairs).flat()
				.map(e => e.toString())
		}

		cards.shuffle().forEach(word => {
			let ref = Math.abs(word);
			put(this.form,
				'figure>img[src=$]<', `/data/word/32/${ref}.jpg`,
				{onclick: e => this.turn(e), id: ref, className: word < 0 ? 'mirror' : ''}
			);
		});
	}
};
